import {Component, EventEmitter, Input, Output} from "@angular/core";
import {BannerService} from "./service/banner.service";
import {NavigationStart} from "@angular/router";
import {Subscription} from "rxjs";
import {Alert} from "../alert/model/alert.model";
import {Banner} from "./model/banner.model";

@Component({
  selector: 'dsmj-banner',
  templateUrl: './banner.component.html'
})
export class BannerComponent {

  /** source type par défaut de l'alerte */
  @Input() id: string = 'default-banner'
  /** true - ajout un effet fade (voir la class.fade)*/
  @Input() fade: boolean = true
  @Input() message: string
  @Input() closeLabel: string

  /** liste des banners actives */
  banners: Banner[] = []
  /** souscription a l'ajout d'alertes */
  messageSubscription!: Subscription
  /** souscription au change de route pour la suppression automatique des alertes actives */
  routeSubscription!: Subscription

  constructor(protected bannerService: BannerService) {
  }

  /** Initialise la gestion des alertes (ajout/suppression) */
  ngOnInit(): void {
    // Observe l'ajout de nouvelle alerte
    this.messageSubscription = this.bannerService.onMessage(this.id)
      .subscribe(banner => {
        // si la nouvelle alerte ne comprend pas message nous l'ignorons,
        // ce cas survient lors du changement de route
        // le changement se faisant nous supprimons les attributs keepAfterRouteChange des alertes en cours
        if (!banner.message && !banner.template) {
          // cherche les alertes avec 'keepAfterRouteChange'
          this.banners = this.banners.filter(x => x.options?.keepAfterRouteChange)

          // supprime l'attribut 'keepAfterRouteChange' des alertes trouvées
          this.banners.forEach(x => delete x.options?.keepAfterRouteChange)
          return
        }

        // ajout des alertes au tampon
        this.banners.push(banner)
      })
  }

  /** execute les unsubscribes */
  ngOnDestroy(): void {
    this.messageSubscription?.unsubscribe()
    this.routeSubscription?.unsubscribe()
  }

  /** supprime l'alerte */
  removeBanner = (banner: Banner): void => {
    console.log('removeAlert :', banner)
    // Check si l'alertes existe toujours
    if (!this.banners.includes(banner)) return

    if (this.fade) {
      // si l'alerte existe lui ajoute l'option 'fade'
      const tmp = this.banners.find(x => x === banner)
      if (tmp) {
        tmp.options!['fade'] = true
      }

      // supprime l'alerte après un temps
      setTimeout(() => {
        this.banners = this.banners.filter(x => x !== banner)
      }, 250)
    } else {
      // supprime l'alerte
      this.banners = this.banners.filter(x => x !== banner)
    }
  }
}
