import {ActivatedRouteSnapshot, Data, NavigationEnd, Router} from "@angular/router";
import {Breadcrumb} from "../model/breadcrumb.model";
import {BehaviorSubject} from "rxjs";
import {Injectable} from "@angular/core";
import {filter} from "rxjs/operators";

@Injectable({
  providedIn: 'root'
})
export class BreadcrumbService {

  // Subject emitting the breadcrumb hierarchy
  private readonly _breadcrumbs$ = new BehaviorSubject<Breadcrumb[]>([]);

  // Observable exposing the breadcrumb hierarchy
  readonly breadcrumbs$ = this._breadcrumbs$.asObservable();

  constructor(private router: Router) {
    this.router.events.pipe(
      // Filter the NavigationEnd events as the breadcrumb is updated only when the route reaches its end
      filter((event) => event instanceof NavigationEnd)
    ).subscribe(event => {
      // Construct the breadcrumb hierarchy
      const root = this.router.routerState.snapshot.root;
      let breadcrumbs: Breadcrumb[] = [];
      this.addBreadcrumb(root, [], breadcrumbs);

      breadcrumbs = this.addHomeBreadcrumb(breadcrumbs)

      // Emit the new hierarchy
      this._breadcrumbs$.next(this.selectLastBreadcrumb(breadcrumbs));
    });
  }

  private selectLastBreadcrumb(breadcrumbs: Breadcrumb[]): Breadcrumb[] {
    breadcrumbs.filter(b => b.selected).forEach(b => b.selected = false)
    if (breadcrumbs.length > 0) {
      breadcrumbs[breadcrumbs.length - 1].selected = true;
    }
    return breadcrumbs;
  }

  private addHomeBreadcrumb(breadcrumbs: Breadcrumb[]): Breadcrumb[] {
    if (breadcrumbs.length > 0 && breadcrumbs[0].url !== '/accueil') {
      const breadcrumb = { label: 'Accueil', url: '/' };
      return [ breadcrumb, ...breadcrumbs ];
    }
    return breadcrumbs;
  }

  private addBreadcrumb(route: ActivatedRouteSnapshot, parentUrl: string[], breadcrumbs: Breadcrumb[]): void {
    if (route) {
      // Construct the route URL
      const routeUrl = parentUrl.concat(route.url.map(url => url.path));

      // Add an element for the current route part
      if (route.data.breadcrumb) {
        const breadcrumb = {
          label: this.getLabel(route.data),
          url: '/' + routeUrl.join('/')
        };
        if (breadcrumbs.map(b => b.url).indexOf(breadcrumb.url) < 0) {
          breadcrumbs.push(breadcrumb);
        }
      }

      // Add another element for the next route part
      this.addBreadcrumb(route.firstChild, routeUrl, breadcrumbs);
    }
  }

  private getLabel(data: Data): string {
    // The breadcrumb can be defined as a static string or as a function to construct the breadcrumb element out of the route data
    return typeof data.breadcrumb === 'function' ? data.breadcrumb(data) : data.breadcrumb;
  }

}
