import { DsmjFormControlDirective } from './form-control.directive';
import { Control } from './control';
import { FormGroup } from '@angular/forms';

export class ControlsGroup {

  /**
   * Retourne la liste des controles gérés par cet objet
   */
  public get get(): Control[] { return this._controls; };

  /**
   * FormGroup du parent
   */
  public _formGroupParent: FormGroup;

  /**
   * Liste des controles gérés par cet objet
   */
  public _controls: Control[] = [];

  /**
   * Etat de desactivation du groupe de contrôles
   */
  protected _disabled: boolean;

  /**
   * Récupère l'état de désactivation du groupe de contrôle
   */
  public get disabled(): boolean {
    return this._disabled;
  }

  /**
   * Affecte une valeur vrai/faux à l'état de désactivation du groupe de contrôle
   *  et modifie le DOM pour lui attribuer le statut défini
   */
  public set disabled(disabled: boolean) {
    this._applytoItems((item: DsmjFormControlDirective) =>
      item.el.nativeElement.disabled = true);
    this._disabled = disabled;
  }

  /**
   * Récupére le tag du groupe de contrôles
   */
  public get tag(): string {
    /** indique si tous les controls ont le même tag */
    let isControlersAreSameTags: boolean = true;
    let lastControlerTag: string = null;
    // vérifie si les controls du group ont le même tag
    this._applyToControls((control: Control) => {
      if(lastControlerTag != null && lastControlerTag !== control.tag)
        isControlersAreSameTags = false;
      lastControlerTag = control.tag;
    });

    // Si les controls ne sont pas de même tag, on retour un tag INPUT
    return isControlersAreSameTags ? lastControlerTag : 'INPUT';
  }

  /**
   * Retourne le type du groupe de contrôle s'il existe,
   *  le type faisant foi est le type du premier champ de contrôle
   */
  public get type(): string | null {

    if(this.get[0] && this.get[0].type)
      return this.get[0].type;

    return 'text';
  }

  constructor(formGroupParent?: FormGroup) {
    this._formGroupParent = formGroupParent;
  }

  /**
   * Définis une liste d'objets de contrôle
   * @param items
   */
  setItems(items: DsmjFormControlDirective[]): this {
    items.forEach((item: DsmjFormControlDirective) => {
      this.addItem(item);
    });
    return this;
  }

  /**
   * Ajoute un nouvel objet de contrôle dans le groupe de contrôles
   * @param item
   */
  addItem(item: DsmjFormControlDirective): this {
    let control: Control = new Control(this._formGroupParent);
    control.item = item;
    this._controls.push(control);
    return this;
  }

  /**
   * Ajoute une classe aux objets de contrôle
   * @param className
   */
  addClass(className: string): void {
    this._applytoItems((item: DsmjFormControlDirective) =>
      item.el.nativeElement.classList.add(className));
  }

  /**
   * Ajoute un attribut et sa valeur aux objets de contrôle
   * @param attribute
   * @param value
   */
  setAttribute(attribute: string, value: string): void {
    this._applytoItems((item: DsmjFormControlDirective) =>
      item.el.nativeElement.setAttribute(attribute, value));
  }

  /**
   * Supprime un attribut des objets de contrôle
   * @param attribute
   */
  removeAttribute(attribute: string): void {
    this._applytoItems((item: DsmjFormControlDirective) =>
      item.el.nativeElement.removeAttribute(attribute));
  }

  /**
   * Retourn si le groupe de contrôle est invalide :
   *  si l'un des contrôles est invalide, l'ensemble du groupe l'est également
   */
  isInvalid(): boolean {
    let result = false;
    // on parcourt chaque objets de contrôle introduit dans ce champs
    this._applyToControls((control) => {
      // et vérifie si un objet de contrôle à au moins une erreur, si c'est le cas: groupe invalide
      if(control.validator.isInvalid())
        result = true;
    });
    return result;
  }

  /**
   * Retourne la liste des messages d'invalidité de tous les contrôles du groupe
   *  et supprime les messages en doublons
   * @returns
   */
  getInvalidMessages(): string[] {

    let messages: string[] = [];
    this._applyToControls((control) => {
      // si le validator n'est pas instancié, il n'est pas utile de traiter l'information
      if(!control.validator)
        return;
      messages = messages.concat(control.validator.errors.filter((i) =>
        messages.indexOf(i) < 0));
    });
    return messages;
  }

  /**
   * Supprime une classe aux objets de contrôle
   * @param className
   */
  removeClass(className: string): void {
    this._applytoItems((item: DsmjFormControlDirective) =>
      item.el.nativeElement.classList.remove(className));
  }

  /**
   * Fonction de callback appliquée à l'ensemble des objets de controls
   *  géré par le groupe de contrôle
   * @param fn
   */
  _applytoItems(fn: Function): void {
    this._applyToControls((control) => {
      let item: DsmjFormControlDirective = control.item;
      fn(item);
    });
  }

  /**
   * Fonction de callback appliquée à l'ensemble des controls
   *  géré par le groupe de contrôle
   * @param fn
   */
  _applyToControls(fn: Function): void {
    this._controls.forEach((control) => {
      fn(control);
    });
  }
}
