import {ComponentFixture, TestBed} from '@angular/core/testing';
import { ControlContainer, FormControl, FormGroup, Validators } from '@angular/forms';
import { Control } from './_base/control';
import { getMockedControlContainer } from '../../helper/spec-helper';

import {
  FormFieldComponent,
  CLASSNAME_SELECT_VALID,
  CLASSNAME_SELECT_INVALID,
  CLASSNAME_INPUT_VALID,
  CLASSNAME_INPUT_INVALID,
  IDNAME_FORM_FIELD,
  IDNAME_FORM_FIELD_CONTROL,
  IDNAME_FORM_FIELD_VALID_MESSAGE,
  IDNAME_FORM_FIELD_INVALID_MESSAGE
} from './form-field.component';
import { DsmjFormControlDirective } from './_base/form-control.directive';

let component: FormFieldComponent;
let fixture: ComponentFixture<FormFieldComponent>;
let element: HTMLElement;
let fg: FormGroup = new FormGroup({
  ['required']: new FormControl(undefined, Validators.required),
});

beforeEach( () => {
  TestBed.configureTestingModule({
    declarations: [ FormFieldComponent ],
    providers: [
      { provide: ControlContainer, useValue: getMockedControlContainer(fg) }
    ]
  })
    .compileComponents();
});

/** */
describe('Quand on instancie le composant', () => {
  beforeEach(() => {
    fixture = TestBed.createComponent(FormFieldComponent);
    component = fixture.componentInstance;
  });
  it('Doit retourner vrai lorsque le composant est instancié', () => {
    expect(component).toBeTruthy();
  });
});

/** */
describe('Quand on désactive le champ', () => {
  beforeEach(() => {
    fixture = TestBed.createComponent(FormFieldComponent);
    component = fixture.componentInstance;
    element = fixture.nativeElement
    component.disabled = true;
    fixture.detectChanges();
  });

  it('Doit retourner vrai lorsque le composant est instancié', () => {
    let test: boolean = true;

    if(component._controls.disabled == false)
      test = false;

    component._controls._applytoItems((item: DsmjFormControlDirective) => {
      if(item.el.nativeElement.disabled == false) {
        test = false;
      }
    });
    expect(test).toBeTruthy();
  });
});

/** */
describe('Quand on ajoute un message de validation', () => {
  beforeEach(() => {
    fixture = TestBed.createComponent(FormFieldComponent);
    component = fixture.componentInstance;
    element = fixture.nativeElement
    component.validMessage = 'Ce champs est valide';
    fixture.detectChanges();
  });


  it('Doit retourner une classe de validation pour chaque controle', () => {
    let test: boolean = true;
    component._controls._applytoItems((item: DsmjFormControlDirective) => {
      let classList: DOMTokenList = item.el.nativeElement.classList;
      if(false == (
        classList.contains(CLASSNAME_SELECT_VALID) ||
        classList.contains(CLASSNAME_INPUT_VALID)
      )) {
        test = false;
      }
    });
    expect(test).toBeTruthy();
  });

  it('Doit retourner un attribut `aria-describedby` pour chaque controle', () => {
    let test: boolean = true;
    component._controls._applytoItems((item: DsmjFormControlDirective) => {
      let hasAttribute: boolean = item.el.nativeElement.hasAttribute('aria-describedBy');
      let ariaDescribedbyValue: string = item.el.nativeElement.get('aria-describedBy');
      if(hasAttribute == false)
        test = false;
      if(ariaDescribedbyValue != IDNAME_FORM_FIELD_VALID_MESSAGE)
        test = false;
    });
    expect(test).toBeTruthy();
  });
});

/** */
describe('Quand le champ est en erreur', () => {

  beforeEach(() => {
    fixture = TestBed.createComponent(FormFieldComponent);
    component = fixture.componentInstance;
    element = fixture.nativeElement
    component.disableValidator = true;
    fixture.detectChanges();
  });

  it('Ne doit pas alimenter control.validator lorsqu\'une erreur se produit', () => {
    let test = true;
    component._controls.get.forEach((control: Control) => {
      if(control.validator)
        return false;
    });
    expect(test).toBeTruthy();
  });

  it('Doit comporter la classe d\'invalidité pour chaque controle', () => {
    let test: boolean = true;
    test = shouldHaveInvalidClassOnControls(component);
    expect(test).toBeTruthy();
  });

  it('Doit retourner un attribut `aria-describedby` pour chaque controle', () => {
    let test: boolean = true;
    component._controls._applytoItems((item: DsmjFormControlDirective) => {
      let hasAttribute: boolean = item.el.nativeElement.hasAttribute('aria-describedBy');
      let ariaDescribedbyValue: string = item.el.nativeElement.get('aria-describedBy');
      if(hasAttribute == false)
        test = false;
      if(ariaDescribedbyValue != IDNAME_FORM_FIELD_VALID_MESSAGE)
        test = false;
    });
    expect(test).toBeTruthy();
  });
});

describe('FormFieldComponent', () => {
  beforeEach(() => {
    fixture = TestBed.createComponent(FormFieldComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });
  it('Doit appliquer l\'état de validité ou d\'invalidité', () => {
    let test: boolean = true;

    component.setInvalidOrValidStatement();

    if(component.disableValidator && component.invalid == true)
      test = false;

    if(component.controls.isInvalid() == true && component.invalid == false)
      test = false;

    if(component.controls.isInvalid() == false && component.invalid == true)
      test = false;

    if(component.valid && shouldHaveValidClassOnControls(component) == false)
      test = false;

    if(component.disableValidator && component.valid && shouldHaveValidClassOnControls(component) == true)
      test = false;

    expect(test).toBeTruthy();
  });

  it('Doit ajouter la classe de validation sur tous les objets de controle', () => {
    let test: boolean;
    component.addControlValidClass();
    test = shouldHaveValidClassOnControls(component);
    expect(test).toBeTruthy();
  });

  it('Doit ajouter la classe d\'invalidation sur tous les objets de controle', () => {
    let test: boolean;
    component.addControlInvalidClass();
    test = shouldHaveInvalidClassOnControls(component);
    expect(test).toBeTruthy();
  });

  it('Doit supprimer la classe de validation sur tous les objets de controle', () => {
    let test: boolean;
    component.removeControlValidClass();
    test = shouldNotHaveValidClassOnControls(component);
    expect(test).toBeTruthy();
  });

  it('Doit supprimer la classe d\'invalidation sur tous les objets de controle', () => {
    let test: boolean;
    component.removeControlInvalidClass();
    test = shouldNotHaveInvalidClassOnControls(component);
    expect(test).toBeTruthy();
  });
});

function shouldHaveValidClassOnControls(component): boolean {
  let test: boolean = true;
  component._controls._applytoItems((item: DsmjFormControlDirective) => {
    let classList: DOMTokenList = item.el.nativeElement.classList;
    if(false == (
      classList.contains(CLASSNAME_SELECT_VALID) ||
      classList.contains(CLASSNAME_INPUT_VALID)
    )) {
      test = false;
    }
  });
  return test;
}

function shouldHaveInvalidClassOnControls(component): boolean {
  let test: boolean = true;
  component._controls._applytoItems((item: DsmjFormControlDirective) => {
    let classList: DOMTokenList = item.el.nativeElement.classList;
    if(false == (
      classList.contains(CLASSNAME_SELECT_INVALID) ||
      classList.contains(CLASSNAME_INPUT_INVALID)
    )) {
      test = false;
    }
  });
  return test;
}

function shouldNotHaveValidClassOnControls(component): boolean {
  let test: boolean = true;
  component._controls._applytoItems((item: DsmjFormControlDirective) => {
    let classList: DOMTokenList = item.el.nativeElement.classList;
    if(true == (
      classList.contains(CLASSNAME_SELECT_INVALID) ||
      classList.contains(CLASSNAME_INPUT_INVALID)
    )) {
      test = false;
    }
  });
  return test;
}

function shouldNotHaveInvalidClassOnControls(component): boolean {
  let test: boolean = true;
  component._controls._applytoItems((item: DsmjFormControlDirective) => {
    let classList: DOMTokenList = item.el.nativeElement.classList;
    if(true == (
      classList.contains(CLASSNAME_SELECT_VALID) ||
      classList.contains(CLASSNAME_INPUT_VALID)
    )) {
      test = false;
    }
  });
  return test;
}
