import { Component, Input, ContentChildren, QueryList, AfterContentInit, Optional } from "@angular/core";
import { ControlContainer, FormGroup } from "@angular/forms";
import { DsmjFormControlDirective } from "./_base/form-control.directive";
import { ControlsGroup } from "./_base/controls-group";
import { v4 as uuidv4 } from 'uuid';


/** Unique id de l'objet */
const UUID: string = uuidv4();

/** Select valide */
export const CLASSNAME_RADIO_VALID: string = 'fr-radio--valid';

/** Select invalide */
export const CLASSNAME_RADIO_INVALID: string = 'fr-radio--error';

/** Select valide */
export const CLASSNAME_SELECT_VALID: string = 'fr-select--valid';

/** Select invalide */
export const CLASSNAME_SELECT_INVALID: string = 'fr-select--error';

/** Input valide */
export const CLASSNAME_INPUT_VALID: string = 'fr-input--valid';

/** Input invalide */
export const CLASSNAME_INPUT_INVALID: string = 'fr-input--error';

/** Container du champ */
export const IDNAME_FORM_FIELD: string = 'dsmj-form-field_' + UUID;

/** Objet de control du champ */
export const IDNAME_FORM_FIELD_CONTROL: string = 'dsmj-form-field_control_' + UUID;

/** Block de texte valide */
export const IDNAME_FORM_FIELD_VALID_MESSAGE: string = 'dsmj-valid-text_' + UUID;

/** Block de texte d'erreur */
export const IDNAME_FORM_FIELD_ERROR_MESSAGE: string = 'dsmj-error-text_' + UUID;

/** Block de texte invalide */
export const IDNAME_FORM_FIELD_INVALID_MESSAGE: string = 'dsmj-invalid-text_' + UUID;

@Component({
  selector: 'dsmj-form-field',
  templateUrl: './form-field.component.html',
  styleUrls: ['./form-field.component.scss']
})
export class FormFieldComponent implements AfterContentInit {

  /**
   * Applique l'état désactivé au champ
   */
  @Input() disabled: boolean = false;

  /**
   *  Permet de désactiver l'état de validation automatique
   */
  @Input() disableValidator: boolean;

  /**
   * Message de validité à afficher
   */
  @Input() validMessage: string;

  /**
   * Message de validité à afficher
   */
  @Input() errorMessage: string;

  /**
   * Ajoute une icone au champ
   */
  @Input() icon: string

  /**
   * Texte de description additionnel
   */
  @Input() hint: string;

  /**
   * Label associé au champ de contrôle
   */
  @Input() label: string;

  /**
   * Attribut for associé au label
   */
  @Input() for: string;

  /**
   * Le champ est obligatoire
   */
  @Input() required: boolean;

  /**
   * Le champ est obligatoire
   */
  @Input() size: 'sm' | 'md' | 'lg' = 'md';

  /**
   * Liste des objets correspondant aux controles associés à ce champ
   */
  @ContentChildren(DsmjFormControlDirective) _itemControls: QueryList<DsmjFormControlDirective>;

  /**
   * Liste des objets correspondant aux controles associés à ce champ au format tableau
   */
  protected itemControls: DsmjFormControlDirective[] = [];

  /**
   * Valeur de l'état d'invalidité
   */
  protected _invalid: boolean;

  /**
   * Liste des messages d'invalidité de tous les contrôles associés au champ
   */
  public invalidMessages: string[] = [];

  /**
   * Group de contrôles affectés au champ
   */
  public _controls: ControlsGroup;

  /**
   * Retourne les controles du champ
   */
  public get controls(): ControlsGroup { return this._controls; }

  /**
   * Set les controles du champs
   */
  public set controls(controls: ControlsGroup) { this._controls = controls; }

  /**
   * Indique si l'état est au statut d'invalidité
   */
  public get invalid(): boolean { return this._invalid; }

  /**
   * Renseigne l'état d'invalidité
   */
  public set invalid(invalid: boolean) { this._invalid = invalid; }

  /**
   * Indique si l'état est au statut de validité
   */
  public get valid(): boolean { return this.validMessage ? true : false; }

  /**
   * Retourne le tag du champ
   */
  public get tag(): string { return this.controls.tag; }

  /**
   * S'il est indiqué Retourne le type du champ
   */
  public get type(): string | null { return this.controls.type; }


  /**
   * Retourne la liste des noms des identifiants
   */
  get idname(): {
    IDNAME_FORM_FIELD,
    IDNAME_FORM_FIELD_CONTROL,
    IDNAME_FORM_FIELD_VALID_MESSAGE,
    IDNAME_FORM_FIELD_ERROR_MESSAGE,
    IDNAME_FORM_FIELD_INVALID_MESSAGE} {
    return {
      'IDNAME_FORM_FIELD': IDNAME_FORM_FIELD,
      'IDNAME_FORM_FIELD_CONTROL': IDNAME_FORM_FIELD_CONTROL,
      'IDNAME_FORM_FIELD_VALID_MESSAGE': IDNAME_FORM_FIELD_VALID_MESSAGE,
      'IDNAME_FORM_FIELD_ERROR_MESSAGE': IDNAME_FORM_FIELD_ERROR_MESSAGE,
      'IDNAME_FORM_FIELD_INVALID_MESSAGE': IDNAME_FORM_FIELD_INVALID_MESSAGE
    }
  }

  constructor(@Optional() public _parentContainer?: ControlContainer) {}

  ngAfterContentInit(): void {

    this.initControlsGroup();

    // Applique l'état de validité de
    if(this.valid)
      this.addControlValidClass();

    // Applique l'état désactivé au champ ou non selon le paramètre
    if(this.disabled)
      this.controls.disabled = true;
  }

  /**
   * Initialisation des groupes de contrôle
   */
  initControlsGroup(): void {

    /** FormGroup parent */
    let formGroupParent = this._parentContainer?.control as FormGroup;

    /** Instancie le groupe d'objets de contrôle de ce champ */
    let items: DsmjFormControlDirective[] = this._itemControls.toArray();

    // initialisation de l'objet group de contrôles
    this.controls = new ControlsGroup(formGroupParent);
    this.controls.setItems(items);

    // initialise les messages d'invalidités
    this.invalidMessages = this.controls.getInvalidMessages();

    // Ecoute les modifications faites sur les objets de controles de ce champ
    this.controls.get.forEach((control) => {
      if(control.formGroup)
        control.formGroup.valueChanges.subscribe(() => this.setInvalidOrValidStatement());
    });
  }

  /**
   * Applique l'état d'invalidité ou de validité du formulaire
   */
  setInvalidOrValidStatement(): void {

    // Si l'état de validation est désactivé, on ne traite plus l'information
    if(this.disableValidator)
      return;

    // Si champ est invalide applique la classe correspondant au statut erreur
    if(this.controls.isInvalid()) {
      this.invalid = true;
      // On ajoute la class correspondant au status de validité
      this.addControlInvalidClass();
      // On supprime la class de contrôle valid
      this.removeControlValidClass()
    }

    // Si le champ est valide supprime la classe correspondant au statut erreur
    else {
      // si le champ est spécifié valide, ajoute la classe de validité du controle
      if(this.valid) {
        this.addControlValidClass();
      }
      // pas d'état valide ou invalide, on supprime l'aria describedBy
      else
        this.controls.removeAttribute('aria-describedby');

      this.invalid = false;
      // On applique la classe correspondant au status de validité
      this.removeControlInvalidClass();
    }
  }

  /**
   * Ajoute une classname `valide` au contrôle
   **/
  addControlValidClass(): void {
    this.controls.setAttribute('aria-describedby', this.idname.IDNAME_FORM_FIELD_VALID_MESSAGE);
    this.controls.addClass(
      this.tag == 'SELECT' ? CLASSNAME_SELECT_VALID :
        this.tag == 'RADIO' ? CLASSNAME_RADIO_INVALID :
        this.tag == 'INPUT' || this.tag == 'TEXTAREA' ? CLASSNAME_INPUT_VALID : '');
  }

  /**
   * Supprime une classname `valide` au contrôle
   **/
  removeControlValidClass(): void {
    this.controls.removeClass(
      this.tag == 'SELECT' ? CLASSNAME_SELECT_VALID :
        this.tag == 'RADIO' ? CLASSNAME_RADIO_INVALID :
        this.tag == 'INPUT' || this.tag == 'TEXTAREA' ? CLASSNAME_INPUT_VALID : ''
    );
  }

  /**
   * Ajoute une classname `invalide` au contrôle
   **/
  addControlInvalidClass(): void {
    this.controls.setAttribute('aria-describedby', this.idname.IDNAME_FORM_FIELD_INVALID_MESSAGE);
    this.controls.addClass(
      this.tag == 'SELECT' ? CLASSNAME_SELECT_INVALID :
        this.tag == 'RADIO' ? CLASSNAME_RADIO_INVALID :
        this.tag == 'INPUT' || this.tag == 'TEXTAREA' ? CLASSNAME_INPUT_INVALID : ''
    );
  }

  /**
   * Ajoute une classname `invalide` au contrôle
   **/
  removeControlInvalidClass(): void {
    this.controls.removeClass(
      this.tag == 'SELECT' ? CLASSNAME_SELECT_INVALID :
        this.tag == 'RADIO' ? CLASSNAME_RADIO_INVALID :
        this.tag == 'INPUT' || this.tag == 'TEXTAREA' ? CLASSNAME_INPUT_INVALID : ''
    );
  }
}
