import {Directive, Input, OnDestroy, OnInit} from "@angular/core";
import {ControlContainer, FormBuilder, FormGroup} from "@angular/forms";
import {Subject} from "rxjs"

@Directive()
export class FormBaseComponent<T> implements OnInit, OnDestroy {
  /** sertà pré alimenter le formGroup. ex : en cas d'édition */
  @Input() data?: T
  /** formGroup parent */
  @Input() formGroup: FormGroup
  /** formGroup du composant à injecté dans le parentContainer.formGroup */
  _formGroup: FormGroup | Function
  /** nom de l'input array dans lequel le formGroup doit être injecté */
  _nameControl?: string
  /** Et déclenché à la destruction du composant. Utiliser pour les takeUntil */
  protected _destroy = new Subject()

  constructor(protected fb: FormBuilder, protected _parentContainer: ControlContainer) {
  }

  ngOnInit(): void {
    // s'il n'y a pas de fromGroup en Input on récupère celui du context parent
    if (!this.formGroup) {
      this.formGroup = this._parentContainer.control as FormGroup
    }
    // sans nameControl aucune possibilité d'injection dans le parent
    if (this._nameControl) {
      this.injectFormGroup()

      // S'il y a des données et que le formArray existe alors on charge les données dans le fg
      if (this.data && this.formGroup?.get(this._nameControl)) {
        this.loadData()
      }
    }
  }

  /**
   * Injection du formGroup du composant dans le formGroup parent
   * @protected
   */
  protected injectFormGroup(): void {
    if (this.formGroup && this._nameControl && typeof this._formGroup !== 'function') {
      this.formGroup.setControl(this._nameControl, this._formGroup)
    }
  }

  /**
   * Alimente le FormGroup du composant et l'alimente avec les données dans les index correspondant
   * les nameInput contenant file sont ignoré, car pressenti comme des données de type invalide pour le controller.
   * @protected
   */
  protected loadData(): void {
    if (this.formGroup && this._nameControl) {
      const fbp = this.formGroup?.get(this._nameControl) as FormGroup
      for (const fbpKey in fbp.controls) {
        if (this.data?.[fbpKey] != undefined) {
          if (!'file'.includes(fbpKey)) {
            fbp.get(fbpKey)?.setValue(this.data?.[fbpKey])
          }
        }
      }
    }
  }

  /**
   * retourne true si un controller de parenté direct correspond au ctx
   * @param ctx
   * @protected
   */
  protected isFGContext(ctx: string): boolean {
    if (!this.formGroup) return false

    return this._findContext(ctx, this.formGroup)
  }

  /**
   * Retourne un boolean selon s'il trouve un controller de parenté direct au ctx
   * @param ctx inputName d'un controller parent direct
   * @param fb FormGroup de recherche
   * @private
   */
  private _findContext(ctx: string, fb: FormGroup): boolean {
    const fbParent = fb?.parent
    if (fbParent == undefined) {
      return false
    }
    for (const parentKey in fbParent.controls) {
      if (parentKey == ctx) {
        if (fbParent?.get(parentKey) == fb) {
          return true
        }
      }
    }
    return this._findContext(ctx, fbParent as FormGroup)
  }

  ngOnDestroy(): void {
    this._destroy.next(1)
  }
}
