import {Directive, EventEmitter, Injectable, Input, OnDestroy, OnInit, Output} from '@angular/core'
import {AbstractControl, ControlContainer, FormGroup, ValidationErrors} from '@angular/forms'
import {BehaviorSubject, Subject} from 'rxjs'
import {takeUntil} from "rxjs/operators";

@Injectable()
@Directive()
export abstract class InputBaseComponent implements OnDestroy {
  /** Set le nameInput de l'input */
  @Input() nameInput: string
  /** Set le label de l'input */
  @Input() name: string = ''
  /** Set le label de l'input */
  @Input() label: string = ''
  /** Set l'id html de l'input */
  @Input() id: string = ''
  /** Si l'input est désactivé */
  @Input() isDisabled?: boolean = undefined

  /** Set la valeur de l'input */
  @Input()
  set value(value: string[] | undefined) {
    if (!value) {
      value = []
    }
    this._value.next(value)
  }
  /** Get la valeur de l'input */
  get value(): string[] {
    return this._value.value
  }

  get fieldTouched(): boolean {
    return this.input.touched
  }

  get fieldInvalid(): boolean {
    return this.input.invalid
  }

  get fieldValid(): boolean {
    return this.input.valid
  }

  /** Renvoie via un EventEmitter la valeur de l'input en cas de changement */
  @Output() hasChange: EventEmitter<any> = new EventEmitter()

  /** Et déclenché à la destruction du composant. Utiliser pour les takeUntil */
  protected _destroy = new Subject()
  /** FormGroup parent */
  formGroup: FormGroup

  _value = new BehaviorSubject(undefined);

  /**
   * Retourne l'input du formGroup selon le nameInput
   */
  get input(): AbstractControl {
    return this.formGroup?.get(this.nameInput)
  }

  /**
   * Retourne le "group", morceau de class css dsfr, si l'input est désactivé ou en erreur
   */
  get group(): string {
    return (this.input?.touched || this.input?.dirty) && this.input?.errors ? 'error' : (this.isDisabled ? 'disabled' : '')
  }

  get hasError(): boolean {
    return !!((this.input?.touched || this.input?.dirty) && this.input?.errors);
  }

  get hasSuccess(): boolean {
    return !!((this.input?.touched || this.input?.dirty) && !this.input?.errors);
  }

  /** Retourne les validateurs ou un objet vide si l'input est désactivé
   * @return AbstractControl
   */
  get validators(): any {
    return this.isDisabled ? {} : this.input?.validator?.({} as AbstractControl)
  }

  constructor(protected _parentContainer: ControlContainer) { }

  init(): void {
    // console.log('---------------------> ', this._parentContainer)
    this.formGroup = this._parentContainer.control as FormGroup

    // si aucun nom set, on construit un nom selon le formGroup parent
    if (this.name == null || this.name == '') {
      this.name = this.buildName()
    }
    // Si aucun id set, on utilise le nom pour valeur
    if (this.id == null || this.id == '') {
      this.id = this.name
    }

    // On initialise la souscription au valueChange pour le hasChange emiter
    if (this.formGroup && this.formGroup?.get(this.nameInput)) {
      const input = this.formGroup.get(this.nameInput)
      input!.valueChanges.pipe(takeUntil(this._destroy)).subscribe(v => {
        this.hasChange.emit(v)
      })
    }
  }

  /**
   * Construit un nom machine pour l'input en se basant sur le formGroup et le nameInput
   */
  buildName(): string {
    let name = ''
    if (!this.formGroup || !this.nameInput) {
      return name
    }
    for (const parent in this.formGroup.controls) {
      if (parent === this.nameInput) {
        name = this._findName(this.formGroup)
      }
    }

    return name + '-' + this.nameInput
  }

  /**
   * Construit un nom machine en se basant sur l'arbre du formGroup
   */
  _findName(fb: FormGroup): string {
    const fbParent = fb?.parent
    for (const parentKey in fbParent?.controls) {
      if (fbParent?.get(parentKey) == fb) {
        const r = this._findName(fbParent as FormGroup)
        return r != '' ? r + '-' + parentKey : parentKey
      }
    }

    return ''
  }

  /**
   * Retourne le nom de la classe du groupe affecté au champ
   * @returns string
   */
  inputGroupClassname(): string {
    return this.group ? 'fr-input-group--' + this.group : undefined;
  }

  /**
   *
   * @returns Retourne le nom de la classe affectée à la description des erreurs
   */
  descErrorIdname(): string {
    return this.id + '-desc-error';
  }

  ngOnDestroy(): void {
    this._destroy.next(1)
  }
}

