import {Directive, ElementRef, HostListener, Input, Renderer2} from '@angular/core'

/**
 * Permet l'ajout d'un tooltip au survol d'un élément
 */
@Directive({
  selector: '[dsmjTooltip]'
})
export class DsmjTooltipDirective {
  /** message à afficher */
  @Input() tooltipTitle?: string;
  /** positionnement autour de l'élément */
  @Input() placement: string = 'bottom';
  /** délai de disparition */
  @Input() delay: number = 500;
  /** élément html de la tooltip - si affiché */
  tooltip?: HTMLElement;
  /** décalage de l'affichage par rapport à l'élément */
  offset = 10;

  constructor(private el: ElementRef, private renderer: Renderer2) { }

  /** affiche la tooltip */
  @HostListener('mouseover') onMouseEnter(): void {
    if (!this.tooltip) { this.show(); }
  }

  /** supprime la tooltip */
  @HostListener('mouseleave') onMouseLeave(): void {
    if (this.tooltip) { this.hide(); }
  }

  /** initialise la construction de l'élément html Tooltip */
  show(): void {
    if(!this.tooltipTitle) {
      return
    }
    this.create();
    this.setPosition();
    this.renderer.addClass(this.tooltip, 'ng-tooltip-show');
  }

  /** initialise de suppression de l'élément html Tooltip */
  hide(): void {
    this.renderer.removeClass(this.tooltip, 'ng-tooltip-show');
    window.setTimeout(() => {
      this.renderer.removeChild(document.body, this.tooltip);
      this.tooltip = undefined;
    }, this.delay);
  }

  /** créé et inject l'élément html Tooltip dans le body */
  create(): void {
    this.tooltip = this.renderer.createElement('span');

    this.renderer.appendChild(
      this.tooltip,
      this.renderer.createText(this.tooltipTitle || '') // textNode
    );

    this.renderer.appendChild(document.body, this.tooltip);

    this.renderer.addClass(this.tooltip, 'ng-tooltip');
    this.renderer.addClass(this.tooltip, `ng-tooltip-${this.placement}`);

    this.renderer.setStyle(this.tooltip, '-webkit-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, '-moz-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, '-o-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, 'transition', `opacity ${this.delay}ms`);
  }

  /** set la position de la tooltip */
  setPosition(): void {
    const hostPos = this.el.nativeElement.getBoundingClientRect();

    const tooltipPos = this.tooltip?.getBoundingClientRect();

    const scrollPos = window.scrollY || document.documentElement.scrollTop || document.body.scrollTop || 0;

    if (tooltipPos == undefined) return;
    let top, left;

    if (this.placement === 'top') {
      top = hostPos.top - tooltipPos.height - this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
    }

    if (this.placement === 'bottom') {
      top = hostPos.bottom + this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2;
    }

    if (this.placement === 'left') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.left - tooltipPos.width - this.offset;
    }

    if (this.placement === 'right') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.right + this.offset;
    }

    this.renderer.setStyle(this.tooltip, 'top', `${top + scrollPos}px`);
    this.renderer.setStyle(this.tooltip, 'left', `${left}px`);
  }
}
