import {Component, forwardRef, Input} from '@angular/core';
import { NG_VALUE_ACCESSOR, ControlValueAccessor } from '@angular/forms';
import {DsfrRadioButton} from "../dsfr-radio-button";

//Incrmental ID to avoid the same id attribute for group
let nextId = 0;

@Component({
  selector: 'dsmj-radio-group',
  templateUrl: './radio-group.component.html',
  styleUrls: ['./radio-group.component.scss'],
  providers: [
    {provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => RadioGroupComponent), multi: true},
  ]
})
export class RadioGroupComponent implements ControlValueAccessor {
  /** radio bon in the group **/
  private _radios: Set<DsfrRadioButton> = new Set<DsfrRadioButton>();
  /** Selected value for the radio group. */
  private _value: any = null;
  /** Disabled status value for the radio group. */
  private _disabled: boolean = false;

  /** Incoming ID of the group **/
  @Input() id: string = '';

  /** DIsplay inline or vertically **/
  @Input() inline: boolean = false;
  /** Légende **/
  @Input() legend: string = '';
  /** Texte de description additionnel pour la lélende **/
  @Input() legendHint: string = '';

  /** Type of status : nothing, succes (validd) or error **/
  @Input() status: 'success' | 'error' | any ;
  /** status message going along with status type **/
  @Input() statusMessage: string = '';
  /**
   * Recommend: make it true when at least one radio button mandatory and want to add '*' with legend text
   */
  @Input() isAsteriskRequired: boolean = false;


  /**
   * Name of the radio group applied to radio input elements.
   *
   * Will be applied to all radio input elements inside the group,
   * unless dsfr-radio's specify names themselves.
   *
   * If not provided, will be generated in the `dsfr-radio-group-xx` format.
   */
  @Input() name = `dsfr-radio-group-${nextId++}`;


  get disabled() { return this._disabled; }
  @Input()
  set disabled(isDisabled: boolean) { this.setDisabledState(isDisabled); }

  /**
   * Value for the radio-group. Should equal the value of the selected radio button if there is
   * a corresponding radio button with a matching value. If there is not such a corresponding
   * radio button, this value persists to be applied in case a new radio button is added with a
   * matching value.
   */
  @Input()
  get value(): any { return this._value; }
  set value(newValue: any) {
    if (this._value !== newValue) {
      // Set this before proceeding to ensure no circular loop occurs with selection.
      this._value = newValue;
    }
  }

  /** Constructor **/
  constructor() { }

  /**
   * Registers a callback to be triggered when the model value changes.
   * Implemented as part of ControlValueAccessor.
   * @param fn Callback to be registered.
   */
  registerOnChange(fn: (value: any) => any): void { this.onChange = fn; }


  /**
   * Registers a callback to be triggered when the control is touched.
   * Implemented as part of ControlValueAccessor.
   * @param fn Callback to be registered.
   */
  registerOnTouched(fn: () => any): void { this.onTouched = fn; }

  /**
   * Sets the disabled state of the control. Implemented as a part of ControlValueAccessor.
   * @param isDisabled Whether the control should be disabled.
   */
  setDisabledState(isDisabled: boolean): void {
    this._disabled = isDisabled;
  }

  /**
   * Write the value of the control.Implemented as a part of ControlValueAccessor.
   * @param value the new value
   */
  writeValue(value: any): void {
    this._value = value;
    this._updateRadiosValue();
  }

  /** The method to be called in order to update ngModel */
  onChange = (_: any) => {};
  /**
   * onTouch function registered via registerOnTouch (ControlValueAccessor).
   * @docs-private
   */
  onTouched = () => {};

  /**
   * Update the value of the radio button
   * @param radio the button radio in the button group
   */
  onRadioChange(radio: DsfrRadioButton) {
    this.writeValue(radio.value);
    this.onChange(radio.value);
  }

  /**
   * Notify radio button when updated
   */
  onRadioValueUpdate() { this._updateRadiosValue(); }

  /** unregister a radio button to the group **/
  unregister(radio: DsfrRadioButton) { this._radios.delete(radio); }
  /** Register a radio button to the group **/
  register(radio: DsfrRadioButton) { this._radios.add(radio); }

  /** Generate the ID attribute for a legend **/
  getLegendId(){
    return this.name + "-legend";
  }

  private _updateRadiosValue() { this._radios.forEach((radio) => radio.updateValue(this._value)); }

  /** Get the class attribute value according to the status type (error, valid) **/
  getStatusClass(){
    if( this.isValid()) {
      return 'fr-valid-text';
    } else if(this.isError()) {
      return 'fr-error-text';
    }
    return "";
  }
  /**
   * Generate status id
   * `{attr.name}-inline-desc-valid` : when inline with status success
   * `{attr.name}-desc` : when vertically with no specific status
   **/
  getStatusId(){
    return this.name + (this.inline ? "-inline-" : "") + "-desc-" +
      (this.isValid() ? "-valid" : this.isError() ? '-error' : '')
  }
  /** Check if a specif status (error or valid) exist **/
  hasStatus(){
    return this.isValid() || this.isError();
  }

  /** Return true if status code is `success` **/
  isValid(){
    return this.status === 'success';
  }

  /** Return true if status code is `error` **/
  isError(){
    return this.status === 'error';
  }

  /** Generate value for aria-labelBy **/
  ariaLabelBy(){
    return this.isError() ? this.getLegendId() + ' ' + this.getStatusId() : null;
  }
}
