import {Component, EventEmitter, Input, OnInit, Output, ViewChild} from '@angular/core';
import {DndErrorManager, DndErrorType} from "./model/dnd-error.model";

@Component({
  selector: 'dsmj-upload-file',
  templateUrl: './upload-file.component.html',
  styleUrls: ['./upload-file.component.scss']
})
export class UploadFileComponent implements OnInit {
  @ViewChild('file', { static: false }) file;

  @Input() files: Set<File | Object> = new Set();
  @Input() allowedExtensions?: Array<string>;
  @Input() title: string = 'Ajouter vos documents'
  @Input() btnLabel: string = 'Choisir vos fichiers'
  @Input() dndLabel: string = 'Glisser/Déposer vos fichiers ici'
  @Input() errorsLabel?: DndErrorManager;
  @Input() noFilesLabel: string = 'Aucun fichier sélectionné'
  @Input() disabled = false;
  @Input() errorTimerLimit?: number = 5000;
  @Input() filesNumberLimit?: number = 10;
  @Input() filesSizeLimit?: number = 10;
  @Output() filesChangeEmitter = new EventEmitter<Set<File | Object>>();

  constructor() { }

  ngOnInit(): void {
    this.defaultErrorManager()
  }

  error: DndErrorType;

  get containsError(): boolean {
    return this.error != null;
  }

  defaultErrorManager(): void {
    if (this.errorsLabel == null) {
      this.errorsLabel = {
        format: 'Format de fichier non supporté',
        size: `Taille de fichier dépasse la limite autorisé de ${this.filesSizeLimit} Mo`,
        number: `Le nombre maximum de fichiers est de ${this.filesNumberLimit}`,
      }
    }
  }

  filesChangeSender(files: Set<File | Object>): void {
    this.filesChangeEmitter.emit(files)
  }

  addFiles(): void {
    this.file.nativeElement.click();
  }

  onFileChangeFromInput(target: EventTarget): void {
    this.onFileChange((target as HTMLInputElement).files);
  }

  onFileChange(pFileList: (File | Object)[] | any): void {
    if (this.disabled) return;
    this.error = null;
    this.checkFileExtension(pFileList, () => {
      this.checkFileNumber(pFileList, () => {
        Object.keys(pFileList).forEach(key => {
          const add = (): void => {
            this.files.add(pFileList[key])
          }
          this.checkFileSize(pFileList[key], add)
        });
      });
    });
    this.filesChangeSender(this.files);
  }

  onFileErrors(err: DndErrorType): void {
    if (this.disabled) return;
    // this.errors = true;
    this.errorTimer(err);
  }

  errorTimer(err: DndErrorType): void {
    this.error = err;
    if (this.errorTimerLimit && this.errorTimerLimit > 0) {
      setTimeout(() => {
        this.error = null;
      }, this.errorTimerLimit)
    }
  }

  private callbackHandler(callback): () => void {
    return (): void => {
      if (callback) callback()
    };
  }

  private checkFileSize(file: File, callback): void {
    const proceedCallback = this.callbackHandler(callback);

    if (this.filesSizeLimit == null) proceedCallback()
    const isValid = (file.size / 1024 / 1024) <= this.filesSizeLimit;
    this.fileChecker(isValid, 'size', proceedCallback);
  }

  private checkFileNumber(files: (File | Object)[], callback): void {
    const proceedCallback = this.callbackHandler(callback);

    if (this.filesNumberLimit == null) proceedCallback()
    const isValid = (this.files?.size || 0) + files.length <= this.filesNumberLimit;
    this.fileChecker(isValid, 'number', proceedCallback);
  }

  private checkFileExtension(files: (File | Object)[], callback): void {
    const proceedCallback = this.callbackHandler(callback);

    console.log('file ', files)
    const isValid = this.validateFiles(files);
    this.fileChecker(isValid, 'format', proceedCallback);
  }

  private fileChecker(isValid: boolean, error: DndErrorType, proceedCallback: () => void): void {
    if (isValid) {
      proceedCallback()
    } else {
      this.onFileErrors(error)
    }
  }

  private validateFiles(files: (File | Object)[]): boolean {
    if (this.allowedExtensions == null) return true;
    return Array.from(files).every(f => this.validateFile(f['name']));
  }

  private validateFile(name: String): boolean {
    if (this.allowedExtensions == null) return true;

    const ext = name.substring(name.lastIndexOf('.') + 1)?.toLowerCase();
    return this.allowedExtensions.map(val => val.toLowerCase()).includes(ext);
  }
}
