import {
  Directive,
  ElementRef,
  HostBinding,
  Input,
  OnDestroy,
  OnInit,
  Optional,
  Renderer2,
  TemplateRef,
} from '@angular/core';
import {AbstractControl, ControlContainer, FormControl, FormGroup, FormGroupDirective, ValidationErrors} from "@angular/forms";
import {nanoid} from 'nanoid'
import {NgIfContext} from "@angular/common";
import {merge, Observable, of, Subscription} from "rxjs";
import {map} from "rxjs/operators";

@Directive({
  selector: '[dsmjFormValidity]',
})
export class DsmjFormValidityDirective implements OnInit, OnDestroy {
  @Input() controlName?: string
  @Input() isContainer: boolean = false;
  @Input() colorSuccess: boolean = false
  @Input() colorError: boolean = true
  @Input() colorInput: boolean = true
  @Input() validationOnTouch: boolean = false
  @Input() template_validity?: TemplateRef<NgIfContext<any>>

  @HostBinding('class.fr-input-group--error') isContainerInvalid: boolean;
  @HostBinding('class.fr-input-group--valid') isContainerValid: boolean;
  @HostBinding('class.fr-input-group--disabled') isContainerDisabled: boolean;

  @HostBinding('class.fr-input--error') isInvalid: boolean;
  @HostBinding('class.fr-input--valid') isValid: boolean;

  formGroup?: FormGroup;
  isDirty?: boolean;
  isTouched?: boolean;

  controlValue$: Observable<any>;
  controlSubscription: Subscription;
  hasSubmitted = false;

  constructor(protected el: ElementRef, protected renderer: Renderer2, @Optional() protected _parentContainer?: ControlContainer) {
  }

  get formControlName(): string {
    return this.getFormControlName() ?? this.controlName
  }

  get formControl(): AbstractControl | undefined {
    const formControlName = this.formControlName;
    return (null != formControlName) ? this.formGroup.controls[formControlName] : undefined;
  }

  ngOnInit(): void {
    if (this.mustHaveFormGroup()) {
      this.formGroup = this._parentContainer?.control as FormGroup
      if (null == this.formGroup) return;

      let formSubmit$ = (<FormGroupDirective>this._parentContainer).ngSubmit.pipe(map((): void => {
        this.hasSubmitted = true;
      }));
      this.controlValue$ =  merge(this.formControl.valueChanges, of(''), formSubmit$ );
      this.controlSubscription = this.controlValue$.subscribe((): void => {
        this.init()
      });
    }
  }

  ngOnDestroy(): void {
    this.controlSubscription?.unsubscribe();
  }

  init(): void {
    this.isDirty = this.isControlDirty();
    this.isTouched = this.isControlTouched();

    if ((this.formControl?.invalid || this.formControl?.valid) && (this.formControl?.dirty || this.hasSubmitted)) {
      this.isContainerInvalid = this.applyErrorColorOnLabel() && this.isControlInvalid();
      this.isContainerValid = this.applySuccessColorOnLabel() && this.isControlValid();
      this.isContainerDisabled = this.isContainer && this.isControlDisabled();

      this.isInvalid = this.applyColorOnInput() && this.isControlInvalid();
      this.isValid = this.applyColorOnInput() && this.isControlValid();
    }
  }

  private mustHaveFormGroup(): boolean {
    return (null != this.formControlName);
  }

  private getFormControlName(): any {
    const element = this.el.nativeElement;
    return element.getAttribute('formControlName');
  }

  private hasValidator(): boolean{
    const control = this.formControl;
    return this.getSyncValidators(control) != null
      || this.getAsyncValidators(control) != null;
  }

  private applyErrorColorOnLabel(): boolean {
    return this.validationOnTouch ?
      this.colorError && this.isContainer && this.isTouched :
      this.colorError && this.isContainer && this.isDirty;
  }

  private applySuccessColorOnLabel() : boolean{
    return this.validationOnTouch ?
      this.colorSuccess && this.isContainer && this.isTouched :
      this.colorSuccess && this.isContainer && this.isDirty;
  }

  private applyColorOnInput(): boolean {
    return this.validationOnTouch ?
      this.colorInput && !this.isContainer && this.isTouched :
      this.colorInput && !this.isContainer && this.isDirty;
  }

  private isControlValid(): boolean {
    return this.hasValidator() && this.formControl?.valid
  }

  private isControlInvalid(): boolean {
    return this.hasValidator() && this.formControl?.invalid
  }

  private isControlDisabled(): boolean {
    return this.hasValidator() && this.formControl?.disabled
  }

  private isControlDirty(): boolean {
    return this.hasValidator() && this.formControl?.dirty
  }

  private isControlTouched(): boolean {
    return this.hasValidator() && this.formControl?.touched
  }


  private getSyncValidators(control: AbstractControl): ValidationErrors | null {
    if(control?.validator) {
      return control.validator({} as AbstractControl);
    }
    return null;
  }

  private getAsyncValidators(control: AbstractControl): Promise<ValidationErrors> | Observable<ValidationErrors> | null {
    if(control?.asyncValidator) {
      return control.asyncValidator({} as AbstractControl);
    }
    return null;
  }

  /** génère un uuid */
  // private override getUuid() {
  private static getUuid(): string {
    let uuid
    while (true) {
      uuid = nanoid(8)
      if (/^(.*[a-zA-Z]+.*)$/.test(uuid)) break
    }
    return uuid
  }
}
