import {Directive, ElementRef, Input, OnDestroy, OnInit, Optional, Renderer2} from "@angular/core";
import {AbstractControl, ControlContainer, FormGroup, FormGroupDirective} from "@angular/forms";
import {merge, Observable, of, Subscription} from "rxjs";
import {map} from "rxjs/operators";

interface ErrorType extends Object {
  required?: string,
  min?: string,
  max?: string,
  minlength?: string,
  maxlength?: string,
  pattern?: string,
  email?: string,
}

@Directive({
  selector: '[dsmjInvalidMessage]'
})
export class DsmjInvalidMessageDirective implements OnInit, OnDestroy {
  @Input() dsmjInvalidMessage: ErrorType;
  @Input() controlName: string;
  control: AbstractControl;
  hasView = false;
  controlValue$: Observable<any>;
  controlSubscription: Subscription;
  hasSubmitted: boolean;

  hydratedMesages = [];

  constructor(
    private _el: ElementRef,
    private render: Renderer2,
    @Optional() private _fg: ControlContainer,
  ) { }

  ngOnInit(): void {
    this.initDefault();
    if (null == this.form) return;
    this.control = this.form.get(this.controlName);
    let formSubmit$ = (<FormGroupDirective>this._fg).ngSubmit.pipe(map(()=>{
      this.hasSubmitted = true;
    }));
    this.controlValue$ =  merge(this.control.valueChanges, of(''), formSubmit$ );
    this.controlSubscription = this.controlValue$.subscribe(() => {
      this.setVisible();
    });
  }

  get form(): FormGroup | null { return this._fg?.formDirective ? (this._fg.formDirective as FormGroupDirective).form : null; }

  private setVisible(): void {
    this.removeMessage();
    if (this.control.invalid && (this.control.dirty || this.hasSubmitted)) {
      if (this.control.invalid) {
        this.showMessage();
      }
    }
  }

  /**
   * <input dsmjValidMessage type="text" name="field"> will not init default messages
   * <input [dsmjInvalidMessage] type="text" name="field"> will init default messages
   * @private
   */
  private initDefault(): void {
    if (this.dsmjInvalidMessage == null)
      this.dsmjInvalidMessage = DsmjInvalidMessageDirective.getDefaultMessages();
  }

  private showMessage(): void {
    const errors = this.createErrors(this.dsmjInvalidMessage);
    const div = this._el.nativeElement;
    if (errors && errors.length > 0) {
      errors.forEach(error => this.render.appendChild(div, error))
    }
  }

  private removeMessage(): void {
    const removeOneChild = (id): void => {
      const p = this._el.nativeElement.querySelector('#'.concat(id))
      this.render.removeChild(this._el.nativeElement, p);
    }
    this.hydratedMesages.forEach(id => removeOneChild(id));
    this.hydratedMesages = [];
  }

  private createErrorParagraph(message, id): HTMLParagraphElement {
    const paragraph = document.createElement('p');
    paragraph.setAttribute('id', id)
    paragraph.classList.add(...['fr-error-text'])
    paragraph.innerText = message;

    return paragraph;
  }

  private createsError(key: string, message: string): HTMLParagraphElement | undefined {
    if (this.control.hasError(key)) {
      const id = this.controlName.concat('-').concat(key);
      this.hydratedMesages.push(id);
      return this.createErrorParagraph(message, id);
    }
  }

  private createErrors(errorMessages: ErrorType): HTMLParagraphElement[] | undefined {
    const keys = Object.keys(errorMessages);
    return keys.filter(k => errorMessages[k] != null)
      .map(key => this.createsError(key, errorMessages[key]))
      .filter(Boolean)
  }

  private static getDefaultMessages(): ErrorType {
    return {
      required: 'Ce champ obligatoire',
      min: 'La valeur du champ est trop basse',
      max: 'La valeur du champ est trop élevé',
      minlength: 'Le nombre de caractère n\'est pas suffisant',
      maxlength: 'Le nombre de caractère dépasse la limite autorisée',
      pattern: 'Le format du champ est invalide',
      email: 'Le format de l\'email est invalide',
    }
  }

  ngOnDestroy(): void {
    this.controlSubscription?.unsubscribe();
  }
}
